@@ -6,23 +6,26 @@ use File::Basename qw(fileparse);
 use File::Spec;
 use Cwd qw(cwd);
 use Test::More;
-use Test::Deep qw(cmp_details deep_diag methods);
+use Test::Deep qw(cmp_details deep_diag methods set);
 use Exporter();
 use strict;
 
 our @ISA=qw(Exporter);
-our @EXPORT=qw(script scriptpath scriptfullpath scriptbasename scriptcode subtestdir rootpath
+our @EXPORT=qw(script scriptpath scriptfullpath scriptbasename scriptcode scripthead 
+	       subtestdir rootpath
 	       as_bool as_list flatten
-	       is_quietly is_loudly cmp_quietly cmp_attrs report report_pass report_fail 
-	       called_from
+	       is_quietly is_loudly cmp_quietly cmp_set_quietly cmp_attrs 
+	       report report_pass report_fail 
+	       called_from group val2idx
 	     );
-our($SCRIPT,$SCRIPTPATH,$SCRIPTBASENAME,$SCRIPTCODE,$ROOTPATH);
+our($SCRIPT,$SCRIPTPATH,$SCRIPTBASENAME,$SCRIPTCODE,$SCRIPTHEAD,$ROOTPATH);
 sub script {$SCRIPT or (($SCRIPT,$SCRIPTPATH)=fileparse($0) and $SCRIPT);}
 sub scriptpath {$SCRIPTPATH or (($SCRIPT,$SCRIPTPATH)=fileparse($0) and $SCRIPTPATH);}
 sub scriptfullpath {$FindBin::Bin}
 sub scriptbasename {$SCRIPTBASENAME or 
 		      (($SCRIPTBASENAME)=fileparse($0,qw(.t)) and $SCRIPTBASENAME);}
 sub scriptcode {$SCRIPTCODE or (($SCRIPTCODE)=script=~/\.(\w+)\.t$/)[0];}
+sub scripthead {$SCRIPTHEAD or (($SCRIPTHEAD)=script=~/^(.*?)\.\d+/);}
 sub subtestdir {File::Spec->catdir(scriptpath,scriptbasename)}
 sub rootpath {$ROOTPATH or $ROOTPATH=cwd}
 
@@ -51,6 +54,11 @@ sub cmp_quietly {
   my($ok,$details)=cmp_details($actual,$correct);
   report_fail($ok,"$label",$file,$line,$details);
 }
+# like cmp_set but reports errors the way we want
+sub cmp_set_quietly {
+  my($actual,$correct,$label,$file,$line)=@_;
+  cmp_quietly($actual,set(@$correct),$label,$file,$line);
+}
 # $actual is object. $correct is HASH of attr=>value pairs wih Test::Deep decorations
 sub cmp_attrs {
   my($actual,$correct,$label,$file,$line)=@_;
@@ -106,18 +114,25 @@ sub called_from {
 ################################################################################
 
 # # TODO: rewrite w/ Hash::AutoHash::MultiValued
-# # group a list by categories returned by sub.
-# # has to be declared before use, because of prototype
-# sub group (&@) {
-#   my($sub,@list)=@_;
-#   my %groups;
-#   for (@list) {
-#     my $group=&$sub($_);
-#     my $members=$groups{$group} || ($groups{$group}=[]);
-#     push(@$members,$_);
-#   }
-#   wantarray? %groups: \%groups;
-# }
+# group a list by categories returned by sub.
+# has to be declared before use, because of prototype
+sub group (&@) {
+  my($sub,@list)=@_;
+  my %groups;
+  for (@list) {
+    my $group=&$sub($_);
+    my $members=$groups{$group} || ($groups{$group}=[]);
+    push(@$members,$_);
+  }
+  wantarray? %groups: \%groups;
+}
+# produce hash mapping each element of list to its position. doesn't worry about duplicates
+sub val2idx {
+  my $i=0;
+  my %val2idx=map {$_=>$i++} @_;
+  wantarray? %val2idx: \%val2idx;
+}
+
 # # like group, but processes elements that are put on list. 
 # # sub should return 2 element list: 1st defines group, 2nd maps the value
 # # has to be declared before use, because of prototype
